#!/usr/bin/env python3
#exonware/xwsystem/shared/__init__.py
"""
Company: eXonware.com
Author: Eng. Muhammad AlShehri
Email: connect@exonware.com
Version: 0.0.1.375
Generation Date: 10-Sep-2025

Shared types and utilities for XWSystem modules.
"""

from .types import (
    ValidationLevel,
    PerformanceLevel,
    AuthType,
    LockType,
    PathType,
    LogLevel,
    OperationResult
)

__all__ = [
    "ValidationLevel",
    "PerformanceLevel", 
    "AuthType",
    "LockType",
    "PathType",
    "LogLevel",
    "OperationResult"
]
